//  Copyright 2004 Garth Gillespie, All Rights Reserved.
// This C++ code is part of Analog Annelid, the creation of Garth Gillespie,
// And may only be used for educational, non-profit purposes.
// For more information, please visit http://www.garthgillespie.com/annelid/ .

// The Apple class represents the apples that the worm must collect to earn
// points.  It is a derivative of Collider and has all of that class's
// necessary features.
//
// Note: The apple spawns a new apple, generates a FlyingNumber of the points
// earned, alters the worms score, and makes the worm grow upon collision.

#ifndef GG_APPLE_H
#define GG_APPLE_H

#include"shared.h"
#include"collider.h"

class Apple:public Collider
{
  public:
    Apple();

    virtual bool update() { frame++; return true; }
    virtual void display(double xmin, double xmax, double ymin, double ymax) const;

    virtual double x() const { return xpos; }
    virtual double y() const { return ypos; }
    virtual double r() const { return INITIAL_RADIUS; }

    virtual void collided(Worm &worm, std::list<Collider*> &colliders, std::list<Visual*> &visuals) const;

  private:
    // determines the point value for collecting the apple at the current frame
    unsigned long int score() const { return (unsigned long int)(1000.0 * double(FRAMES_per_SEGMENT) / double(frame)); }

    double xpos;
    double ypos;
    int frame;
};

#endif
