//  Copyright 2004 Garth Gillespie, All Rights Reserved.
// This C++ code is part of Analog Annelid, the creation of Garth Gillespie,
// And may only be used for educational, non-profit purposes.
// For more information, please visit http://www.garthgillespie.com/annelid/ .

// The Callider class represents anything in the game world with which the
// worm can collide.
//
// update() is called to let the object change over time, and to alert the
// program when the object should be removed in cases where it has not been
// hit, such as when a PowerUp disappears.
//
// display() displays the object, its parameters define the viewable portion
// of game space.
//
// x(), y(), and r() return the objects position and size (radius).
//
// collided() allows the object to affect the worm, generate new colliders,
// create visuals, etc. upon collision.

#ifndef GG_COLLIDER_H
#define GG_COLLIDER_H

#include"visual.h"
#include"worm.h"

#include<list>

class Collider
{
  public:
    virtual ~Collider() {}

    virtual bool update() = 0;
    virtual void display(double xmin, double xmax, double ymin, double ymax) const = 0;

    virtual double x() const = 0;
    virtual double y() const = 0;
    virtual double r() const = 0;

    virtual void collided(Worm &worm, std::list<Collider*> &colliders, std::list<Visual*> &visuals) const = 0;
};

#endif
