//  Copyright 2004 Garth Gillespie, All Rights Reserved.
// This C++ code is part of Analog Annelid, the creation of Garth Gillespie,
// And may only be used for educational, non-profit purposes.
// For more information, please visit http://www.garthgillespie.com/annelid/ .

#include"fonttexture.h"

#include<fstream>
#include<string.h>
#include<GL/gl.h>

bool FontTexture::load(const char *filename)
{
  unload();

  // acquire display lists
  if(!(displayList = glGenLists(128)))
    return false;

  // variables for string manipulation of 'filename'
  int filename_len = strlen(filename);
  char *fullname = new char[filename_len+5];

  // load in font metrics, and generate display lists based on them
  strcpy(fullname,filename);
  strcpy(fullname+filename_len,".dat");
  std::ifstream fin(fullname);
  for(int y = 7, i = 0; y >= 0; y--)
    for(int x = 0; x < 16; x++, i++)
    {
      // if the metric file ends, abort loading
      if(!fin.good())
      {
        fin.close();
        delete [] fullname;
        unload();
        return false;
      }

      int width = fin.get();
      glNewList(displayList + i, GL_COMPILE);
        // display the appropriate section of the texture
        glBegin(GL_QUADS);
          glTexCoord2f(double(x  )/16.0,double(y  )/8.0); glVertex3f(0,0,0);
          glTexCoord2f(double(x+1)/16.0,double(y  )/8.0); glVertex3f(1,0,0);
          glTexCoord2f(double(x+1)/16.0,double(y+1)/8.0); glVertex3f(1,1,0);
          glTexCoord2f(double(x  )/16.0,double(y+1)/8.0); glVertex3f(0,1,0);
        glEnd();
        // advance the line by the width of the character
        glTranslatef(width/16.0,0,0);
      glEndList();
    }
  fin.close();

  // ASCII 10 '\n' new line
  glNewList(displayList+10,GL_COMPILE);
    glPopMatrix();
    glTranslatef(0,-1,0);
    glPushMatrix();
  glEndList();

/*// removed so "\r\n" will work as expected
  // ASCII 13 '\r' carriage return
  glNewList(displayList+13,GL_COMPILE);
    glPopMatrix();
    glTranslatef(0,-1,0);
    glPushMatrix();
  glEndList();
*/

  // load the texture
  strcpy(fullname+filename_len,".tga");
  if(!Texture::load(fullname))
  {
    delete [] fullname;
    unload();
    return false;
  }

  delete [] fullname;
  return true;
}

void FontTexture::unload()
{
  if(displayList)
  {
    // release the display lists and forget them
    glDeleteLists(displayList,128);
    displayList = 0;
  }
  Texture::unload();
}

void FontTexture::display(const char *text) const
{
  // if no texture is loaded, there's nothing to display
  if(!texName) return;

  // bind the texture and display it by calling the appropriate display list for each glyph
  glBindTexture(GL_TEXTURE_2D,texName);
  glListBase(displayList);
  glPushMatrix(); // preserve the model matrix
  glCallLists(strlen(text),GL_BYTE,text);
  glPopMatrix();
}
