//  Copyright 2004 Garth Gillespie, All Rights Reserved.
// This C++ code is part of Analog Annelid, the creation of Garth Gillespie,
// And may only be used for educational, non-profit purposes.
// For more information, please visit http://www.garthgillespie.com/annelid/ .

// FontTexture extends Texture and alters its functionality significantly to
// allow direct rendering of character strings.
//
// load() takes a suffix-less value to which it appends ".tga" to determine
// the image file and ".dat" to determine the font metrics file. Currently,
// the image is assumed to be 256*128, defining 128 individual 16*16 glyphs,
// ordered horizontally left to right, vertically top to bottom, and left-
// aligned within their bounding boxes. The metrics file is byte array of
// the horizontal widths of the glyphs.
//
// display() takes a character array and renders each associated glyph in
// order left to right, one after another.  Special consideration is taken for
// the '\n' character, which advances the rendering down one line and back to
// the same left justifaction.  The first glyph is rendered over the
// coordinates (0,0),(1,0),(1,1),(0,1).
//
// In other respects, this class functions like Texture.

#ifndef GG_FONTTEXTURE_H
#define GG_FONTTEXTURE_H

#include"texture.h"

class FontTexture: private Texture
{
  public:
    FontTexture():Texture(),displayList(0) {}
    ~FontTexture() { unload(); }

    bool load(const char *); // "font"
    void unload();

    void display(const char *) const;

  private:
    unsigned int displayList;
};

#endif
