//  Copyright 2004 Garth Gillespie, All Rights Reserved.
// This C++ code is part of Analog Annelid, the creation of Garth Gillespie,
// And may only be used for educational, non-profit purposes.
// For more information, please visit http://www.garthgillespie.com/annelid/ .

#include"highscore.h"
#include"shared.h"
#include"main.h"

#include<GL/gl.h>
#include<string.h>

#include<fstream>
#include<sstream>
#include<string>

// use the given score and name, or a default name if no name is given
HighScoreList::HighScore::HighScore(unsigned int sin, const char *nin)
:score(sin) { strncpy(name,(strlen(nin)?nin:DEFAULT_NAME),MAX_NAMESIZE); name[MAX_NAMESIZE] = '\0'; }

HighScoreList::HighScoreList(const char *filename)
{
  // store filename
  int len = strlen(filename);
  fname = new char[len+1];
  strcpy(fname,filename);

  // open the file
  std::ifstream fin(fname,std::ios::binary);
  if(!fin)
    return;

  // read the number of scores
  unsigned char count;
  fin.read((char*)(&count),sizeof(count));
  if(count > MAX_HIGHSCORES) // this should not happen, but just in case
    count = MAX_HIGHSCORES;

  // read each score and add it to the list
  for(unsigned int i = 0; i < count; i++)
  {
    HighScore highscore;
    fin.read((char*)(&highscore),sizeof(HighScore));
    if(fin)
      highscores.push_back(highscore);
    else
      break;
  }

  // clean up
  fin.close();
}

unsigned int HighScoreList::place(unsigned int score)
{
  // find the first score that is less than the given score
  unsigned int i = 1;
  for(std::list<HighScore>::iterator hs = highscores.begin();
      hs != highscores.end(); hs++)
  {
    if(score > (*hs).score)
      return i;
    i++;
  }

  // if there is still space in the list, the score get last place
  return (i <= MAX_HIGHSCORES) ? i : 0;
}

void HighScoreList::add(unsigned int score, const char *name)
{
  // find the first score that is less than the given score
  for(std::list<HighScore>::iterator hs = highscores.begin();
      hs != highscores.end(); hs++)
    if(score > (*hs).score)
    {
      highscores.insert(hs,HighScore(score,name));

      // if the list is full, last place gets bumped
      if(highscores.size() > MAX_HIGHSCORES)
        highscores.pop_back();
      return;
    }

  // if there is still space in the list, the score gets last place
  if(highscores.size() < MAX_HIGHSCORES)
    highscores.push_back(HighScore(score,name));
}

void HighScoreList::display() //const
{
  glPushMatrix();
  glTranslatef( (H_RESOLUTION/16.0-17.0)/2.0,                  // centered horizontally. glyphs are 16x16
               -(V_RESOLUTION/16.0-highscores.size()-1.0)/2.0, // centered vertically. glyphs are 16x16
               0);
  font_texture.display("High Scores:\n");

  // add all of the names and a bit of formatting to a single string
  char names[(MAX_NAMESIZE+2)*MAX_HIGHSCORES+1] = {'\0'}; // just enough if every name is filled and maximum size
  for(std::list<HighScore>::iterator hs = highscores.begin();
      hs != highscores.end(); hs++)
  {
    strcat(names,(*hs).name);
    strcat(names,":\n");
  }

  // display the string of names in a single column
  glPushMatrix();
  font_texture.display(names);
  glPopMatrix();

  // slide to the right for the column of scores
  glTranslatef(17.0,0,0);
  for(std::list<HighScore>::iterator hs = highscores.begin();
      hs != highscores.end(); hs++)
  {
    // copy the score to a string
    std::ostringstream sout;
    sout << (*hs).score;

    // display the string
    glPushMatrix();
    glTranslatef(-double(sout.str().length())/2.0,0,0); // right justify. numbers in the selected font have a width of 8px
    font_texture.display(sout.str().c_str());
    glPopMatrix();
    glTranslatef(0,-1,0);
  }

  glPopMatrix();
}

void HighScoreList::write()
{
  // open the file
  std::ofstream fout(fname,std::ios::binary);
  if(!fout)
    return;

  // write the score count
  unsigned char count = highscores.size();
  fout.write((char*)(&count),sizeof(count));

  // write each score
  for(std::list<HighScore>::iterator hs = highscores.begin();
      hs != highscores.end(); hs++)
    fout.write((char*)(&(*hs)),sizeof(HighScore));

  fout.close();
}
