//  Copyright 2004 Garth Gillespie, All Rights Reserved.
// This C++ code is part of Analog Annelid, the creation of Garth Gillespie,
// And may only be used for educational, non-profit purposes.
// For more information, please visit http://www.garthgillespie.com/annelid/ .

// The HighScoreList class manages the high score list for the game, both for
// storage and while the game is running.
//
// The constructor takes the name of the high score data file and reads the
// data from it to populate the list.
//
// place() returns the placement for a particular score.
// add() adds a score to the list.
// display() displays the list center screen.
// write() writes the list back to the data file that was read.
// close() cleans up allocated memory.

#ifndef GG_HIGHSCORE_H
#define GG_HIGHSCORE_H

#include<list>

#define MAX_HIGHSCORES 16
#define MAX_NAMESIZE   15
#define DEFAULT_NAME   "Analog Annelid"

class HighScoreList
{
  public:
    HighScoreList(const char *filename);

    unsigned int place(unsigned int score); // returns 0 for no placement
    void add(unsigned int score, const char *name);
    void display(); // const;
    void write();
    void close() { highscores.clear(); delete fname; }

  private:
    struct HighScore
    {
      HighScore() {}
      HighScore(unsigned int,const char *);
      unsigned long int score;
      char name[MAX_NAMESIZE+1];
    };

    std::list<HighScore> highscores;

    char *fname;
};

#endif
