//  Copyright 2004 Garth Gillespie, All Rights Reserved.
// This C++ code is part of Analog Annelid, the creation of Garth Gillespie,
// And may only be used for educational, non-profit purposes.
// For more information, please visit http://www.garthgillespie.com/annelid/ .

#include"powerup.h"
#include"main.h"
#include"mixer.h"

#include<stdlib.h> // rand(),RAND_MAX


PowerUp::PowerUp()
:xpos(double(H_RESOLUTION)*RANDOM_FLOAT()),ypos(double(V_RESOLUTION)*RANDOM_FLOAT()),frame(0),visualp(0)
{
  // check to see if a powerup (or vortex) should spawn, and if so, which
  double typevalue = RANDOM_FLOAT();
  if(typevalue < POWERUP_FREQUENCY)
  {
    type = effectFast;
    visualp = new SpinFasterVisual(x(),y(),r(),&segment_texture);
    return;
  }

  typevalue -= POWERUP_FREQUENCY;
  if(typevalue < POWERUP_FREQUENCY)
  {
    type = effectSlow;
    visualp = new SpinSlowerVisual(x(),y(),r(),&segment_texture);
    return;
  }

  typevalue -= POWERUP_FREQUENCY;
  if(typevalue < POWERUP_FREQUENCY)
  {
    type = effectThick;
    visualp = new GrowerVisual(x(),y(),r(),&segment_texture);
    return;
  }

  typevalue -= POWERUP_FREQUENCY;
  if(typevalue < POWERUP_FREQUENCY)
  {
    type = effectThin;
    visualp = new ShrinkerVisual(x(),y(),r(),&segment_texture);
    return;
  }

  typevalue -= POWERUP_FREQUENCY;
  if(typevalue < VORTEX_FREQUENCY)
  {
    // determine the vortex's effect
    typevalue = RANDOM_FLOAT();
    if(typevalue < 0.25)
    {
      type = effectReverse;
      visualp = new RotatorVisual(x(),y(),r(),&powerup_texture);
    }
    else if(typevalue < 0.5)
    {
      type = effectJitter;
      visualp = new RotatorVisual(x(),y(),r(),&powerup_texture);
    }
    else if(typevalue < 0.75)
    {
      type = effectSwap;
      visualp = new RotatorVisual(x(),y(),r(),&powerup_texture);
    }
    else // typevalue < 1.0
    {
      type = effectDeform;
      visualp = new RotatorVisual(x(),y(),r(),&powerup_texture);
    }
    return;
  }

  // no powerup spawned
  type = effectNull;
}

void PowerUp::collided(Worm &worm, std::list<Collider*> &colliders, std::list<Visual*> &visuals) const
{
  char *sound[9] = {
    "",           // effectNull
    "speed.wav",  // effectFast
    "slow.wav",   // effectSlow
    "grow.wav",   // effectThick
    "shrink.wav", // effectThin
    "vortex.wav", // effectReverse
    "vortex.wav", // effectJitter
    "vortex.wav", // effectSwap
    "vortex.wav"  // effectDeform
  };

  Mixer_add(sound[int(type)]);
  visuals.push_back(new GrowAndFadeVisual(visualp));
  worm.affect(type);
}
