//  Copyright 2004 Garth Gillespie, All Rights Reserved.
// This C++ code is part of Analog Annelid, the creation of Garth Gillespie,
// And may only be used for educational, non-profit purposes.
// For more information, please visit http://www.garthgillespie.com/annelid/ .

// The PowerUp class represents gameplay-altering powerups that the worm can
// collect.  It is a derivative of Collider and has all of its necessary
// features.
//
// The constructor will spawn a PowerUp at a random location with a random
// type.
//
// valid() will determine whether the PowerUp has a type or not.  Null
// PowerUps are placeholders for when no PowerUp should be spawned and are
// thus not valid.

#ifndef GG_POWERUP_H
#define GG_POWERUP_H

#include"shared.h"
#include"collider.h"

class PowerUp:public Collider
{
  public:
    PowerUp(); // random location and type

    virtual bool update() { visualp->update(); return (++frame > POWERUP_FRAMES) ? false : true; }
    virtual void display(double xmin, double xmax, double ymin, double ymax) const
    { visualp->display(xmin,xmax,ymin,ymax); }

    virtual double x() const { return xpos; }
    virtual double y() const { return ypos; }
    virtual double r() const { return INITIAL_RADIUS; }

    virtual void collided(Worm &worm, std::list<Collider*> &colliders, std::list<Visual*> &visuals) const;

    bool valid() const { return type != effectNull; }

  private:
    EffectType type;
    double xpos;
    double ypos;
    int frame;
    Visual *visualp;
};

#endif
