//  Copyright 2004 Garth Gillespie, All Rights Reserved.
// This C++ code is part of Analog Annelid, the creation of Garth Gillespie,
// And may only be used for educational, non-profit purposes.
// For more information, please visit http://www.garthgillespie.com/annelid/ .

#ifndef GG_SHARED_H
#define GG_SHARED_H

#define VERSION_STRING     "1.0b"

// Game Altering Defines (change these):
#define H_RESOLUTION       384
#define V_RESOLUTION       H_RESOLUTION
#define MOUSE_SENSITIVITY  0.5
#define FRAMES_per_SECOND  60
#define DEGREES_OF_FREEDOM 24
#define INITIAL_RADIUS     8.0
#define MAX_SEGMENT_RADIUS (INITIAL_RADIUS*4.0)
#define MIN_SEGMENT_RADIUS (INITIAL_RADIUS*0.375)
#define SEGMENTS_per_APPLE 4
#define STARTING_SEGMENTS  0
#define COLLISION_FACTOR   0.75
#define POWERUP_FREQUENCY  0.001
#define VORTEX_FREQUENCY   0.002
#define POWERUP_FRAMES     (5*FRAMES_per_SECOND)
#define EFFECT_MIN         1.15
#define EFFECT_RANGE       0.15
#define EFFECT_DURATION    (20*FRAMES_per_SECOND)
#define JITTER_RANGE       (PI/10.0)
#define DEFORM_FACTOR      0.04


// Debugging-related Defines (define or undef these):
#undef DEBUG_NO_GRABBING
#undef DEBUG_EXTRA_KEYS


// Calculated Defines (these should probably be left alone):
#define PI                 3.1415926535897932384626433832795


// Derived Defines (these should probably be left alone):
#define TICKS_per_FRAME    (1000/FRAMES_per_SECOND)
#define STEERING_ANGLE     (2.0*PI/DEGREES_OF_FREEDOM)
#define FRAMES_per_SEGMENT (DEGREES_OF_FREEDOM/6)
#define STARTING_SPEED     ((1.5*MIN_SEGMENT_RADIUS+0.5*INITIAL_RADIUS)/FRAMES_per_SEGMENT)
// MAX_SPEED is calculated in worm.cpp to disallow gaps for current worm size
// MIN_SPEED is the MAX_SPEED for MIN_SEGMENT_RADIUS
#define MIN_SPEED          (2.0*MIN_SEGMENT_RADIUS/FRAMES_per_SEGMENT)



// A Macro
#define RANDOM_FLOAT() (double(rand())/(RAND_MAX+1.0))


// An Enum
enum EffectType {
  effectNull,
  effectFast,
  effectSlow,
  effectThick,
  effectThin,
  effectReverse,
  effectJitter, // timed
  effectSwap,   // timed
  effectDeform  // timed
};

#endif
