//  Copyright 2004 Garth Gillespie, All Rights Reserved.
// This C++ code is part of Analog Annelid, the creation of Garth Gillespie,
// And may only be used for educational, non-profit purposes.
// For more information, please visit http://www.garthgillespie.com/annelid/ .

#include"targa.h"

#include <fstream>
#include <string.h> // memcpy()

#undef OUTPUT_ERRORS
#undef OUTPUT_DEBUG

//FIXME: pay more attention to the header, particularly origin
//FIXME: convert paletted images

unsigned char * getTGA(const char* filename, unsigned int &width, unsigned int &height, unsigned int &bpp)
{
  // open the file
  std::ifstream fin(filename,std::ios::binary);
  if(!fin)
  {
#ifdef OUTPUT_ERRORS
    std::cerr << "getTGA: \"" << filename << "\" does not exist." << std::endl;
#endif
    return 0;
  }

  // read in the header
  unsigned char head[18]; // header
  fin.read((char*)(head),sizeof(head));
  if(!fin)
  {
#ifdef OUTPUT_ERRORS
    std::cerr << "getTGA: \"" << filename << "\"'s header data could not be read." << std::endl;
#endif
    fin.close();
    return 0;
  }

  // check header for appropriate image format
  if(head[2] != 2 && head[2] != 10)
  {
#ifdef OUTPUT_ERRORS
    std::cerr << "getTGA: \"" << filename << "\" has an invalid format." << std::endl;
#endif
    fin.close();
    return 0;
  }
  for(int i = 0; i < 12; i++)
  {
    if(i == 2)
      i++;
    if(head[i])
    {
#ifdef OUTPUT_ERRORS
      std::cerr << "getTGA: \"" << filename << "\" has an invalid format." << std::endl;
#endif
      fin.close();
      return 0;
    }
  }

  // get width and height and compensate for endian-ness
  width = head[13] * 256 + head[12];
  height= head[15] * 256 + head[14];
  // get bits per pixel and convert to bytes
  bpp = head[16] / 8;

  // check for invalid dimensions
  if(width <= 0 || height <= 0
     || (bpp != 3 && bpp != 4))
  {
#ifdef OUTPUT_ERRORS
    std::cerr << "getTGA: \"" << filename << "\"'s has invalid size or depth." << std::endl;
#endif
    fin.close();
    return 0;
  }

  // calculate the byte count of the image data
  unsigned int count = width * height * bpp;

  // allocate memory
  unsigned char *data = new unsigned char[count];
  if(!data)
  {
#ifdef OUTPUT_ERRORS
    std::cerr << "getTGA: Not enough memory for \"" << filename << "\"." << std::endl;
#endif
    fin.close();
    return 0;
  }

  // read in data
  if(head[2] == 2)
    fin.read((char*)(data),count);
  else // head[2] == 10, RLE
    for(unsigned int offset = 0; offset < count;)
    {
      unsigned char packet;
      if(!fin.read((char*)(&packet),1))
        break;
      unsigned int run = (packet & 0x7f) + 1; // length of the next pixel run
      if(packet & 0x80) // repeated pixels
      {
        unsigned char pixel[bpp];
        if(!fin.read((char*)(pixel),bpp))
          break;
        for(unsigned int i = 0; i < run; i++, offset += bpp)
          memcpy(data+offset,pixel,bpp);
      }
      else // varied pixels
      {
        if(!fin.read((char*)(data+offset),run*bpp))
          break;
        offset += run*bpp;
      }
    }
  if(!fin)
  {
#ifdef OUTPUT_ERRORS
    std::cerr << "getTGA: \"" << filename << "\"'s data is corrupted." << std::endl;
#endif
    delete [] data;
    fin.close();
    return 0;
  }

#ifdef OUTPUT_DEBUG
  std::cerr << "getTGA: \"" << filename << "\" read at: " << width << " x " << height << " x " << bpp << "." << std::endl;
#endif

  fin.close();
  return data;
}

unsigned char * convertBGR_RGB(unsigned char* data, unsigned int width, unsigned int height, unsigned int bpp)
{
  // calculate the byte count of the image data
  unsigned int count = width * height * bpp;

  // alter the data a pixel at a time
  for (unsigned int i = 0; i < count; i += bpp)
  {
    unsigned int temp = data[i];
    data[i] = data[i+2];
    data[i+2] = temp;
  }

  return data;
}
