//  Copyright 2004 Garth Gillespie, All Rights Reserved.
// This C++ code is part of Analog Annelid, the creation of Garth Gillespie,
// And may only be used for educational, non-profit purposes.
// For more information, please visit http://www.garthgillespie.com/annelid/ .

#include"texture.h"
#include"targa.h"

#include<GL/gl.h>

void Texture::unload()
{
  if(texName) // there is a loaded texture
  {
    // release the texture name and forget it
    glDeleteTextures(1,&texName);
    texName = 0;
  }
}

bool Texture::load(const char *filename)
{
  // unload any existing texture
  unload();

  unsigned int w,h,b;  // image metrics
  unsigned char *data; // pixel data

  // read in the pixel data or return an error
  if( !(data = getTGA(filename,w,h,b)) )
    return false;

  // generate a unique texture name and store it
  glGenTextures(1,&texName);

  // setup the gl environment to receive the texture
  glBindTexture(GL_TEXTURE_2D,texName);
  glPixelStorei(GL_UNPACK_ALIGNMENT,1);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);

  // send the image metrics and pixel data
  glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA4,w,h,0,(b==4)?GL_BGRA:GL_BGR,GL_UNSIGNED_BYTE,data);

  // clean up temporary allocated data
  delete [] data;

  return true;
}

void Texture::display() const
{
  // if no texture is loaded, there's nothing to display
  if(!texName) return;

  // bind the texture and display it as specified
  glBindTexture(GL_TEXTURE_2D,texName);
  glBegin(GL_QUADS);
    glTexCoord2f(0,0); glVertex3f(0,0,0);
    glTexCoord2f(1,0); glVertex3f(1,0,0);
    glTexCoord2f(1,1); glVertex3f(1,1,0);
    glTexCoord2f(0,1); glVertex3f(0,1,0);
  glEnd();
}
