//  Copyright 2004 Garth Gillespie, All Rights Reserved.
// This C++ code is part of Analog Annelid, the creation of Garth Gillespie,
// And may only be used for educational, non-profit purposes.
// For more information, please visit http://www.garthgillespie.com/annelid/ .

// This generic texture class reads in a targa format image (.tga).
// The image dimensions (width and height) must be multiples of two,
// Dimensions are not checked, though, and incorrect dimensions will produce
// undefined behavior.
//
// display() maps the texture to (0,0),(1,0),(1,1),(0,1) in the same
// orientation as the original image file.  To reduce the number of calls to
// glEnable() and glDisable(), display() assumes that texture mapping is
// enabled in the OpenGL environment.
//
// Note: Texture objects are not meant to be copied, since their resources are
// released when a the object is destroyed.  One instance should be generated
// early on, and then pointers to that object should be used until it is no
// longer needed.

#ifndef GG_TEXTURE_H
#define GG_TEXTURE_H

class Texture
{
  public:
    Texture():texName(0) {}
    ~Texture() { unload(); } // note: copies will unload for all objects

    bool load(const char *); // "image.tga"
    void unload();

    void display() const;

  protected:
    unsigned int texName; // unique identifier for this texture
};

#endif
